@@ -10,8 +10,6 @@
 
 #define TEST_DEBUG 0
 
-#define HAVE_GB 0
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -19,12 +17,9 @@
 /* For ctime() */
 #include <time.h>
 
-#include <ms-ole.h>
-#include <ms-ole-summary.h>
-
-#if HAVE_GB
-#	include <libgb.h>
-#endif
+#include <libole2/ms-ole.h>
+#include <libole2/ms-ole-vba.h>
+#include <libole2/ms-ole-summary.h>
 
 #define BIFF_TYPES_FILE    "biff-types.h"
 
@@ -45,8 +40,6 @@ static GPtrArray *biff_types   = NULL;
 
 static char *cur_dir = NULL;
 
-static void dump_vba (MsOle *f);
-
 static void
 read_types (char *fname, GPtrArray **types)
 {
@@ -167,7 +160,7 @@ list_commands ()
 }
 
 static void
-syntax_error(char *err)
+syntax_error (char *err)
 {
 	if (err) {
 		printf("Error: '%s'\n",err);
@@ -177,6 +170,7 @@ syntax_error(char *err)
 	printf ("Sytax:\n");
 	printf (" ole <ole-file> [-i] [commands...]\n\n");
 	printf (" -i: Interactive, queries for fresh commands\n\n");
+
 	list_commands ();
 	exit(1);
 }
@@ -796,10 +790,81 @@ do_copyout (MsOle *ole)
 	} while (from);
 }
 
-int main (int argc, char **argv)
+static void
+dump_vba (MsOle *f)
+{
+	const char *vbapath = "/_VBA_PROJECT_CUR";
+	char **dir;
+	MsOleStream *s;
+	MsOleStat    st;
+	char *txt;
+	int   i;
+	int   module_count;
+
+	if (ms_ole_stat (&st, f, vbapath, "") != MS_OLE_ERR_OK ||
+	    st.type == MsOleStreamT) {
+		printf ("No valid VBA found\n");
+		return;
+	}
+
+	if (test_stream_open (&s, f, vbapath, "PROJECT", 'r') !=
+	    MS_OLE_ERR_OK) {
+		printf ("No project file... wierd\n");
+	} else {
+		txt = g_new (guint8, s->size);
+		if (!s->read_copy (s, txt, s->size))
+			printf ("Failed to read project stream\n");
+		else {
+			printf ("----------\n");
+			printf ("Project file:\n");
+			printf ("%s", txt);
+			printf ("----------\n");
+		}
+		ms_ole_stream_close (&s);
+		g_free (txt);
+	}
+
+	txt = g_strconcat (vbapath, "/VBA", NULL);
+	if (ms_ole_directory (&dir, f, txt) != MS_OLE_ERR_OK) {
+		printf ("No VBA subdirectory found");
+		g_free (txt);
+		return;
+	}
+
+	module_count = 0;
+	for (i = 0; dir [i]; i++) {
+		if (test_stream_open (&s, f, txt, dir[i], 'r') !=
+		    MS_OLE_ERR_OK)
+			printf ("Error opening %s\n", dir [i]);
+		else {
+			MsOleVba *vba;
+
+			vba = ms_ole_vba_open (s);
+			if (!vba)
+				g_warning ("Stream '%s' not VBA", dir [i]);
+			else {
+				module_count++;
+
+				while (!ms_ole_vba_eof (vba))
+					printf ("%c", ms_ole_vba_getc (vba));
+			}
+			ms_ole_vba_close (vba);
+
+			ms_ole_stream_close (&s);
+		}
+	}
+
+	if (!module_count)
+		printf ("Strange no modules found\n");
+
+	g_free (txt);
+}
+
+int
+main (int argc, char **argv)
 {
 	MsOle *ole;
-	int lp,exit=0,interact=0;
+	int lp, exit = 0, interact = 0;
 	char *buffer = g_new (char, 1024) ;
 
 	if (argc<2)
@@ -845,43 +910,46 @@ int main (int argc, char **argv)
 		if (!interact)
 			printf ("Command : '%s'\n", arg_data[0]);
 		ptr = arg_data[arg_cur++];
-		if (g_strcasecmp(ptr, "ls")==0) {
+		if (!ptr)
+			continue;
+
+		if (g_strcasecmp (ptr, "ls") == 0)
 			list_files (ole);
-		} else if (g_strcasecmp(ptr, "cd")==0)
+		else if (g_strcasecmp (ptr, "cd") == 0)
 			enter_dir (ole);
-		else if (g_strcasecmp(ptr, "dump")==0)
+		else if (g_strcasecmp (ptr, "dump") == 0)
 			do_dump (ole);
-		else if (g_strcasecmp(ptr, "biff")==0)
+		else if (g_strcasecmp (ptr, "biff") == 0)
 			do_biff (ole);
-		else if (g_strcasecmp(ptr, "biffraw")==0)
+		else if (g_strcasecmp (ptr, "biffraw") == 0)
 			do_biff_raw (ole);
-		else if (g_strcasecmp(ptr, "get")==0)
+		else if (g_strcasecmp (ptr, "get") == 0)
 			do_get (ole);
-		else if (g_strcasecmp(ptr, "put")==0)
+		else if (g_strcasecmp (ptr, "put") == 0)
 			do_put (ole);
-		else if (g_strcasecmp(ptr, "copyin")==0)
+		else if (g_strcasecmp (ptr, "copyin") == 0)
 			do_copyin (ole);
-		else if (g_strcasecmp(ptr, "copyout")==0)
+		else if (g_strcasecmp (ptr, "copyout") == 0)
 			do_copyout (ole);
-		else if (g_strcasecmp(ptr, "summary")==0)
+		else if (g_strcasecmp (ptr, "summary") == 0)
 			do_summary (ole);
-		else if (g_strcasecmp(ptr, "docsummary")==0)
+		else if (g_strcasecmp (ptr, "docsummary") == 0)
 			do_docsummary (ole);
-		else if (g_strcasecmp(ptr, "debug")==0)
+		else if (g_strcasecmp (ptr, "debug") == 0)
 			ms_ole_debug (ole, 1);
-		else if (g_strcasecmp(ptr, "tree")==0)
+		else if (g_strcasecmp (ptr, "tree") == 0)
 			ms_ole_debug (ole, 2);
-		else if (g_strcasecmp(ptr, "vba")==0)
+		else if (g_strcasecmp (ptr, "vba") == 0)
 			dump_vba (ole);
-		else if (g_strcasecmp(ptr,"help")==0 ||
-			   g_strcasecmp(ptr,"?")==0 ||
-			   g_strcasecmp(ptr,"info")==0 ||
-			   g_strcasecmp(ptr,"man")==0)
+		else if (g_strcasecmp (ptr,"help") == 0 ||
+			 g_strcasecmp (ptr,"?") == 0 ||
+			 g_strcasecmp (ptr,"info") == 0 ||
+			 g_strcasecmp (ptr,"man") == 0)
 			list_commands ();
-		else if (g_strcasecmp(ptr,"exit")==0 ||
-			   g_strcasecmp(ptr,"quit")==0 ||
-			   g_strcasecmp(ptr,"q")==0 ||
-			   g_strcasecmp(ptr,"bye")==0)
+		else if (g_strcasecmp (ptr,"exit") == 0 ||
+			 g_strcasecmp (ptr,"quit") == 0 ||
+			 g_strcasecmp (ptr,"q") == 0 ||
+			 g_strcasecmp (ptr,"bye") == 0)
 			exit = 1;
 	}
 	while (!exit && interact);
@@ -889,359 +957,3 @@ int main (int argc, char **argv)
 	ms_ole_destroy (&ole);
 	return 0;
 }
-
-
-static void
-print_bin (guint16 dt)
-{
-	int i;
-	
-	printf ("|");
-	for (i = 15; i >= 0; i--) {
-		if (dt & (1 << i))
-			printf ("1");
-		else
-			printf ("0");
-		if (i == 8)
-			printf ("|");
-	}
-	printf ("|");
-}
-
-static guint16
-read_binary (char *txt)
-{
-	int i, dt = 0;
-	
-	g_assert (strlen (txt) > 16);
-
-	for (i = 16; i >= 0; i--) {
-		if (*txt == '|') {
-			txt++;
-			continue;
-		}
-
-		dt = dt << 1;
-		if (*txt++ == '1')
-			dt |= 0x1;
-	}
-	return dt;
-}
-
-/*
- * lzw, arc like compression.
- */
-static GArray *
-decompress_vba (guint8 *data, guint32 len, gboolean debug)
-{
-#define BUF_SIZE    4096 /* a bottleneck */
-	guint8   buffer[BUF_SIZE];
-	guint8  *ptr;
-	guint32  pos;
-	GArray  *ans = g_array_new (FALSE, FALSE, 1);
-
-	for (pos = 0; pos < BUF_SIZE; pos++)
-		buffer[pos] = ' ';
-
-	if (debug) {
-		printf ("My compressed stream:\n");
-		ms_ole_dump (data, len);
-	} else
-		printf ("Uncompressed data\n");
-
-	ptr = data;
-	pos = 0;
-
-	while (ptr < data + len) {
-		guint8 hdr = *ptr++;
-		int    shift;
-
-		for (shift = 0x01; shift < 0x100; shift = shift<<1) {
-			if (pos == 4096) {
-				if (debug) {
-					printf ("\nSomething extremely odd"
-						" happens after 4096 bytes 0x%x\n\n",
-						MS_OLE_GET_GUINT16 (ptr));
-					ms_ole_dump (ptr, len - (ptr - data));
-				}
-				ptr+=2;
-				hdr = *ptr++;
-				pos = 0;
-				shift = 0x01;
-			}
-			if (hdr & shift) {
-				guint16 dt = MS_OLE_GET_GUINT16 (ptr);
-				int i, back, len, shft;
-
-				if (pos <= 64)
-					shft = 10;
-				else if (pos <= 128)
-					shft = 9;
-				else if (pos <= 256)
-					shft = 8;
-				else if (pos <= 512)
-					shft = 7;
-				else if (pos <= 1024)
-					shft = 6;
-				else if (pos <= 2048)
-					shft = 5;
-				else
-					shft = 4;
-
-				back = (dt >> shft) + 1;
-				len = 0;
-				for (i = 0; i < shft; i++)
-					len |= dt & (0x1 << i);
-				len += 3;
-
-				if (debug) {
-					printf ("|match 0x%x (%d,%d) >> %d = %d, %d| pos = %d |\n",
-						dt, (dt>>8), (dt&0xff), shft, back, len, pos);
-				/* Perhaps dt & SHIFT = dist. to end of run */
-					print_bin (dt);
-					printf ("\n");
-				}
-				for (i = 0; i < len; i++) {
-					guint8 c;
-					guint32 srcpos = (BUF_SIZE + (pos%BUF_SIZE)) - back;
-
-					if (srcpos >= BUF_SIZE)
-						srcpos-= BUF_SIZE;
-
-					g_assert (srcpos >= 0);
-					g_assert (srcpos < BUF_SIZE);
-					c = buffer [srcpos];
-					buffer [pos++ % BUF_SIZE] = c;
-					g_array_append_val (ans, c);
-					printf ("%c", c);
-					}
-				ptr += 2;
-			} else {
-				buffer [pos++ % BUF_SIZE] = *ptr;
-				g_array_append_val (ans, *ptr);
-				printf ("%c", *ptr++);
-			}
-		}
-	}
-	
-	{
-		char c = '\0';
-		g_array_append_val (ans, c);
-	}
-	return ans;
-}
-
-static void
-try_parse_vba (GArray *d)
-{
-#if HAVE_GB
-	GBModule const *module;
-	GBParseError error;
-	guint8 *data = d->data;
-	guint32 len  = d->len;
-
-	/* ugly but I'm in a hurry hackout Attribute=""\r\n*/
-	while (len > 0) {
-		if (*data == '"')
-			break;
-		data++; len--;
-	}
-	
-	while (len > 0) {
-		if (*data == '"')
-			break;
-		data++; len--;
-	}
-
-	data+=2;
-
-	if (len <= 0) {
-		g_warning ("Just a tag");
-		return;
-	}
-	gb_init ();
-
-	module = gb_parse_stream (data, len, &error);
-	
-	if (module == NULL)
-		/* FIXME : Move error into error_result */
-		fprintf (stderr, "Error on Line %d\n", error.line);
-	gb_shutdown ();
-
-#else
-	g_warning ("unimplemented");
-#endif
-}
-
-static void
-dump_vba_module (MsOle *f, const char *path, const char *name)
-{
-	MsOleStream *s;
-
-	if (test_stream_open (&s, f, path, name, 'r') !=
-	    MS_OLE_ERR_OK) {
-		printf ("Strange: can't open '%s%s'\n", path, name);
-		return;
-	}
-
-	{ /* A better guess ? */
-		guint8  *data, *ptr;
-		guint32  offset;
-		
-		data = g_new (guint8, s->size);
-		if (!s->read_copy (s, data, s->size)) {
-			printf ("Strange: failed read of module '%s%s'\n", path, name);
-			return;
-		}
-		
-		if (s->size < 0xd4) {
-			printf ("Too small\n");
-			return;
-		}
-
-		offset = MS_OLE_GET_GUINT32 (data + 0xd0);
-
-		if (s->size < offset + 3) {
-			printf ("Too small for offset 0x%x\n", offset);
-			return;
-		}
-
-		printf ("Offset is 0x%x\n", offset);
-
-		/*
-		 * Need to decode the table correctly to extract to 0xd0
-		 * 0x20 - 0x8 + 0xd0 == assesed2 offset; prolly just like
-		 * summary stream format.
-		 */
-		{ /* Less crude but not working */
-			guint8 *ptr = data + offset;
-			GArray *a;
-			guint32 one;
-			guint16 len;
-
-			len = MS_OLE_GET_GUINT16 (ptr + 1);
-
-			one = MS_OLE_GET_GUINT8  (ptr);
-
-			printf ("Length 0x%x, digit (?) 0x%x\n", len, one);
-
-			len = len & ~0xb000;
-
-			a = decompress_vba (ptr + 3, len,
-					    (arg_data [arg_cur] != NULL));
-
-			try_parse_vba (a);
-			g_array_free (a, TRUE);
-		}
-	}
-
-	/* Very, very, very, very, very crude :-) but working */
-/*	{
-		guint8  *data, *ptr;
-		guint32  i;
-		
-		data = g_new (guint8, s->size);
-		if (!s->read_copy (s, data, s->size)) {
-			printf ("Strange: failed read of module '%s%s'\n", path, name);
-			return;
-		}
-		
-		ptr = data;
-		i   = 0;
-		do {
-			if (!g_strncasecmp (ptr, "Attribut", 8)) {
-				GArray *a;
-				int     len;
-				guint32 w;
-				guint16 x;
-
-				len = s->size - (ptr - data);
-				x = MS_OLE_GET_GUINT16 (ptr - 3);
-				w = MS_OLE_GET_GUINT32 (ptr - 7);
-
-				printf ("Length 0x%x, len = 0x%x guess 0x%x\n", len, x, w);
-
-				a = decompress_vba (ptr - 1, len,
-						    (arg_data [arg_cur] != NULL));
-				try_parse_vba (a);
-				g_array_free (a, TRUE);
-			}
-			ptr++; i++;
-		} while (i < s->size - 10);
-
-		g_free (data);
-		}*/
-	
-	ms_ole_stream_close (&s);
-}
-
-/* Hack - leave it here for now */
-static void
-dump_vba (MsOle *f)
-{
-	const char *vbapath = "/_VBA_PROJECT_CUR";
-	char **dir;
-	MsOleStream *s;
-	MsOleStat    st;
-	char *txt;
-	int   i;
-	int   module_count;
-
-	if (ms_ole_stat (&st, f, vbapath, "") != MS_OLE_ERR_OK ||
-	    st.type == MsOleStreamT) {
-		printf ("No valid VBA found\n");
-		return;
-	}
-
-	if (test_stream_open (&s, f, vbapath, "PROJECT", 'r') !=
-	    MS_OLE_ERR_OK) {
-		printf ("No project file... wierd\n");
-	} else {
-		txt = g_new (guint8, s->size);
-		if (!s->read_copy (s, txt, s->size))
-			printf ("Failed to read project stream\n");
-		else {
-			printf ("----------\n");
-			printf ("Project file:\n");
-			printf ("%s", txt);
-			printf ("----------\n");
-		}
-		ms_ole_stream_close (&s);
-	}
-
-	txt = g_strconcat (vbapath, "/VBA", NULL);
-	if (ms_ole_directory (&dir, f, txt) != MS_OLE_ERR_OK) {
-		printf ("No VBA subdirectory found");
-		g_free (txt);
-		return;
-	}
-
-	module_count = 0;
-	for (i = 0; dir [i]; i++) {
-		if (test_stream_open (&s, f, txt, dir[i], 'r') !=
-		    MS_OLE_ERR_OK)
-			printf ("Error opening %s\n", dir [i]);
-		else {
-			const guint8 gid  [16] = { 0x1,  0x16, 0x1,  0x0,
-						   0x6,  0xb6, 0x0,  0xff,
-						   0xff, 0x1,  0x1,  0x0,
-						   0x0,  0x0,  0x0,  0xff };
-			guint8 data [16];
-
-			s->read_copy (s, data, 16);
-			ms_ole_stream_close (&s);
-
-			if (!memcmp (gid, data, 16)) {
-				printf ("%s is VBA module %d\n", dir [i],
-					module_count++);
-				printf ("----------\n");
-				dump_vba_module (f, txt, dir[i]);
-				printf ("----------\n");
-			}
-		}
-	}
-	if (!module_count)
-		printf ("Strange no modules found\n");
-
-	g_free (txt);
-}
